package com.wordpress.salaboy.examples;

import bitronix.tm.TransactionManagerServices;
import bitronix.tm.resource.jdbc.PoolingDataSource;
import java.sql.SQLException;
import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.*;
import org.drools.io.impl.ClassPathResource;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.process.*;
import org.junit.Assert;
import org.junit.Test;
import static org.junit.Assert.*;

import java.util.Map;
import java.util.Properties;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import org.drools.SystemEventListenerFactory;
import org.drools.base.MapGlobalResolver;
import org.drools.compiler.BPMN2ProcessFactory;
import org.drools.compiler.ProcessBuilderFactory;
import org.drools.marshalling.impl.ProcessMarshallerFactory;
import org.drools.persistence.jpa.JPAKnowledgeService;
import org.drools.runtime.Environment;
import org.drools.runtime.EnvironmentName;
import org.drools.runtime.KnowledgeSessionConfiguration;
import org.jbpm.bpmn2.BPMN2ProcessProviderImpl;
import org.jbpm.marshalling.impl.ProcessMarshallerFactoryServiceImpl;
import org.jbpm.process.audit.JPAProcessInstanceDbLog;

import org.jbpm.process.audit.JPAWorkingMemoryDbLogger;
import org.jbpm.process.audit.ProcessInstanceLog;
import org.jbpm.process.builder.ProcessBuilderFactoryServiceImpl;
import org.jbpm.process.instance.ProcessRuntimeFactoryServiceImpl;
import org.jbpm.process.workitem.wsht.WSHumanTaskHandler;
import org.jbpm.task.User;
import org.jbpm.task.service.TaskService;
import org.jbpm.task.service.TaskServiceSession;
import org.jbpm.task.service.mina.MinaTaskServer;

public class ProcessTaskTest {
    private static StatefulKnowledgeSession ksession;

    public ProcessTaskTest() {
    }

    @Test
    public void humanTasksTest() throws InterruptedException, SQLException {

		ProcessBuilderFactory.setProcessBuilderFactoryService(new ProcessBuilderFactoryServiceImpl());
		ProcessMarshallerFactory.setProcessMarshallerFactoryService(new ProcessMarshallerFactoryServiceImpl());
		ProcessRuntimeFactory.setProcessRuntimeFactoryService(new ProcessRuntimeFactoryServiceImpl());
		BPMN2ProcessFactory.setBPMN2ProcessProvider(new BPMN2ProcessProviderImpl());

		PoolingDataSource ds = new PoolingDataSource();
		ds.setUniqueName("oracleTestDS");
		ds.setClassName("oracle.jdbc.xa.client.OracleXADataSource");
		ds.setMaxPoolSize(3);
		ds.setAllowLocalTransactions(true);
		ds.getDriverProperties().put("user", "");
		ds.getDriverProperties().put("password", "");
		ds.getDriverProperties().put("URL", "jdbc:oracle:thin:@21:mxxm");
		ds.init();

		// create the entity manager factory and register it in the environment
		EntityManagerFactory emf = Persistence.createEntityManagerFactory( "org.jbpm.persistence.jpa" );
		System.out.println("emf = " + emf);

		Environment env = KnowledgeBaseFactory.newEnvironment();
		env.set( EnvironmentName.ENTITY_MANAGER_FACTORY, emf );
		env.set( EnvironmentName.TRANSACTION_MANAGER, TransactionManagerServices.getTransactionManager() );
        env.set( EnvironmentName.GLOBALS, new MapGlobalResolver() );
		System.out.println("environment = " + env);

		TaskService taskService = new TaskService(emf, SystemEventListenerFactory.getSystemEventListener());

		TaskServiceSession taskSession = taskService.createSession();
		taskSession.addUser(new User("Admin"));
		//taskSession.addUser(new User("kris"));

		/* Start Mina server for HT*/
		//MinaTaskServer server = new MinaTaskServer(taskService);
		//Thread thread = new Thread(server);
		//thread.start();
		//System.out.println("Server started ...");

		KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
		kbuilder.add(new ClassPathResource("EmergencyServiceSimple.bpmn"), ResourceType.BPMN2);
		KnowledgeBuilderErrors errors = kbuilder.getErrors();
        if (errors.size() > 0) {
            for (KnowledgeBuilderError error : errors) {
                System.out.println(error.getMessage());
            }
            return;
        }

        KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase();
        kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());

		Properties properties = new Properties();
		// because of JPA add:
		properties.put("drools.processInstanceManagerFactory",
						"org.jbpm.persistence.processinstance.JPAProcessInstanceManagerFactory");
		properties.put("drools.processSignalManagerFactory",
				"org.jbpm.persistence.processinstance.JPASignalManagerFactory");

		KnowledgeSessionConfiguration config = KnowledgeBaseFactory.newKnowledgeSessionConfiguration(properties);
		ksession = JPAKnowledgeService.newStatefulKnowledgeSession(kbase, config, env ); // EnvironmentFactory.newEnvironment()

		int sessionId = ksession.getId();

        MyAutomaticHumanSimulatorWorkItemHandler humanActivitiesSimHandler = new MyAutomaticHumanSimulatorWorkItemHandler();
        ksession.getWorkItemManager().registerWorkItemHandler("Human Task", humanActivitiesSimHandler);
		//ksession.getWorkItemManager().registerWorkItemHandler("Human Task", new WSHumanTaskHandler());

		JPAWorkingMemoryDbLogger logger = new JPAWorkingMemoryDbLogger(ksession);

		// start process -> it runs till RuleTask
        WorkflowProcessInstance process = (WorkflowProcessInstance) ksession.startProcess("com.wordpress.salaboy.bpmn2.SimpleEmergencyService");
		System.out.println("proces info puvodni " + process);
		WorkflowProcessInstance processInstance = (WorkflowProcessInstance) ksession.getProcessInstance( process.getId() );
		System.out.println("number of actual nodes " + processInstance.getNodeInstances().size());

		//System.out.println("Completing the first Activity !");
        //Complete the first human activity
        //humanActivitiesSimHandler.completeWorkItem(null);

		 //I need to call the FireAllRules method because I have a RuleTask in my business process
        ksession.fireAllRules();
        System.out.println("session info " + ksession.getId());

		//System.out.println("Completing the second Activity !");
        //Complete the second human activity
        //humanActivitiesSimHandler.completeWorkItem(null);

		// now the process should be finished

        // Is the process completed?
		ksession = JPAKnowledgeService.loadStatefulKnowledgeSession( sessionId, kbase, config, env );
		processInstance = (WorkflowProcessInstance) ksession.getProcessInstance( process.getId() );
		System.out.println("proces info should be null: " + processInstance);
		// process is finished
		assertNull(processInstance);

		JPAProcessInstanceDbLog log = new JPAProcessInstanceDbLog();
		ProcessInstanceLog finishedProcessInstance = log.findProcessInstance(process.getId());

        Assert.assertNotNull(finishedProcessInstance.getEnd());

		System.out.println("end of test");
    }
}

class MyNonAutomaticHumanSimulatorWorkItemHandler implements WorkItemHandler {
    private WorkItemManager workItemManager;
    private long workItemId;

    public void executeWorkItem(WorkItem workItem, WorkItemManager workItemManager) {
        this.workItemId = workItem.getId();
        this.workItemManager = workItemManager;
        System.out.println("Map of Parameters = " + workItem.getParameters());
    }

    public void abortWorkItem(WorkItem workItem, WorkItemManager workItemManager) {

    }

    public void completeWorkItem(Map<String, Object> parameters) {
		System.out.println("workItemId to finish is: " + this.workItemId);
        this.workItemManager.completeWorkItem(this.workItemId, parameters);

    }

}

class MyAutomaticHumanSimulatorWorkItemHandler implements WorkItemHandler {

    public void executeWorkItem(WorkItem workItem, WorkItemManager workItemManager) {
        System.out.println("Map of Parameters = " + workItem.getParameters());
        workItemManager.completeWorkItem(workItem.getId(), null);
    }

    public void abortWorkItem(WorkItem workItem, WorkItemManager workItemManager) {

    }


}
